// Copyright (c) Microsoft Corporation. All rights reserved.

function OnFinish(selProj, selObj)
{
	try
	{
		var strProjectPath = wizard.FindSymbol("PROJECT_PATH");
		var strProjectName = wizard.FindSymbol("PROJECT_NAME");

		var bEmptyProject = wizard.FindSymbol("EMPTY_PROJECT");

		selProj = CreateProject(strProjectName, strProjectPath);
		selProj.Object.Keyword = "Win32Proj";

		AddCommonConfig(selProj, strProjectName);
		var strAppType = GetAppType();
		AddSpecificConfig(selProj, strProjectName, bEmptyProject, strAppType);

		SetupFilters(selProj);
		//Adds another filter
		var group = selProj.Object.AddFilter("Documentation");
		group.Filter = "txt;;chi;chm;doxy";

		//if (!bEmptyProject)
		//{
			if (IsDBCSCharSet(wizard))
				wizard.AddSymbol("ABOUTBOX_FONT_SIZE", "9"); //VS-supported codedepages that require FontSize 9
			else
				wizard.AddSymbol("ABOUTBOX_FONT_SIZE", "8"); //all the rest and unsupported codepages: Fontsize 8 
			
			var Pch = wizard.FindSymbol("PRE_COMPILED_HEADER");

			if ((strAppType == "LIB" || ((strAppType == "CONSOLE") && 
		                !wizard.FindSymbol("SUPPORT_MFC") && !wizard.FindSymbol("SUPPORT_ATL"))) && !Pch) 
			{
				AddFilesToProjectWithInfFile(selProj, strProjectName);
				SetNoPchSettings(selProj);
			}
			else
			{
				AddFilesToProjectWithInfFile(selProj, strProjectName);
				SetNoPchSettings(selProj);
			}

			if (strAppType == "CONSOLE")
			{
				var config = selProj.Object.Configurations("Debug");
				var LinkTool = config.Tools("VCLinkerTool");
				if (wizard.FindSymbol("IS_EXPRESS_SKU"))
				{
    					LinkTool.AdditionalDependencies = "kernel32.lib $(NoInherit)";
    				}

				config = selProj.Object.Configurations("Release");
				LinkTool = config.Tools("VCLinkerTool");
				if (wizard.FindSymbol("IS_EXPRESS_SKU"))
				{
					LinkTool.AdditionalDependencies = "kernel32.lib $(NoInherit)";
    				}
			}

		//}
		selProj.Object.Save();
	}
	catch(e)
	{
		if (e.description.length != 0)
			SetErrorInfo(e);
		return e.number
	}
}

function SetFileProperties(projfile, strName)
{
	return false;
}

function GetTargetName(strName, strProjectName, strResPath, strHelpPath)
{
	try
	{
		var strTarget = strName;

		if (strName == "Docs\\readme.txt")
			strTarget = "Docs\\ReadMe.txt";
		if (strName == "dllmain.h")
			strTarget = strProjectName + ".h";
		if (strName == "import.cpp" ||strName == "export.cpp")
		{
			strTarget = strProjectName + ".cpp";

		}
		if (strName == "Docs\\guide_c.chm" ||strName == "Docs\\guide.chm")
		{
			strTarget = "Docs\\Guide.chm";

		}
		if (strName == "Docs\\guide_c.chi" ||strName == "Docs\\guide.chi")
		{
			strTarget = "Docs\\Guide.chi";

		}
		if (strName == "Doxyfile")
			strTarget = strProjectName +".doxy";
		return strTarget; 
	}
	catch(e)
	{
		throw e;
	}
}

function GetAppType()
{
	try
	{
		var strAppType = "";
		if (wizard.FindSymbol("DELED_EXPORT"))
			strAppType = "DLL";
		else
		{
			if (wizard.FindSymbol("DELED_IMPORT"))
				strAppType = "DLL";
		}
		return strAppType;
	}
	catch(e)
	{
		throw e;
	}
}

function AddSpecificConfig(proj, strProjectName, bEmptyProject, strAppType)
{
	try
	{
		var config = proj.Object.Configurations("Debug");
		config.CharacterSet = charSetMBCS;

		if (strAppType == "LIB")
			config.ConfigurationType = typeStaticLibrary;
		else if (strAppType == "DLL")
			config.ConfigurationType = typeDynamicLibrary;

		var CLTool = config.Tools("VCCLCompilerTool");
	//	CLTool.PrecompiledHeaderFile = "$(OutDir)/" + strProjectName + ".pch";

		CLTool.RuntimeLibrary = rtMultiThreadedDebugDLL;

		var strDefines = CLTool.PreprocessorDefinitions;
		if (strDefines != "") strDefines += ";";
		strDefines += GetPlatformDefine(config);
		strDefines += "_DEBUG";

		switch(strAppType)
		{
			case "DLL":
				strDefines += ";_WINDOWS;_USRDLL;BUILD_DLL;";
				var strExports = wizard.FindSymbol("UPPER_CASE_SAFE_PROJECT_IDENTIFIER_NAME") + "_EXPORTS";
				strDefines += strExports;
				break;
			default:
				break;
		}

		CLTool.PreprocessorDefinitions = strDefines;
		if (bEmptyProject)
			CLTool.UsePrecompiledHeader = pchNone;

		CLTool.DebugInformationFormat = debugEditAndContinue;

		if (strAppType != "LIB")
		{
			var LinkTool = config.Tools("VCLinkerTool");
			LinkTool.GenerateDebugInformation = true;
			LinkTool.LinkIncremental = linkIncrementalYes;
			LinkTool.ModuleDefinitionFile = "ExportTable.def";

			if (strAppType == "DLL")
				LinkTool.SubSystem = subSystemWindows;
		}

		config = proj.Object.Configurations.Item("Release");
		config.CharacterSet = charSetMBCS;

		if (strAppType == "LIB")
			config.ConfigurationType = typeStaticLibrary;
		else if (strAppType == "DLL")
			config.ConfigurationType = typeDynamicLibrary;

		var CLTool = config.Tools("VCCLCompilerTool");

		CLTool.RuntimeLibrary = rtMultiThreadedDLL;

		var strDefines = CLTool.PreprocessorDefinitions;
		if (strDefines != "") strDefines += ";";
		strDefines += GetPlatformDefine(config);
		strDefines += "NDEBUG";
		if (bEmptyProject)
			CLTool.UsePrecompiledHeader = pchNone;

		CLTool.DebugInformationFormat = debugEnabled;

		switch(strAppType)
		{
			case "DLL":
				strDefines += ";_WINDOWS;_USRDLL;BUILD_DLL;";
				var strExports = wizard.FindSymbol("UPPER_CASE_SAFE_PROJECT_IDENTIFIER_NAME") + "_EXPORTS";
				strDefines += strExports;
				break;
			default:
				break;
		}

		CLTool.PreprocessorDefinitions = strDefines;

		if (strAppType != "LIB")
		{
			var LinkTool = config.Tools("VCLinkerTool");
			LinkTool.GenerateDebugInformation = true;
			LinkTool.LinkIncremental = linkIncrementalNo;
			LinkTool.ModuleDefinitionFile = "ExportTable.def";

			if (strAppType == "DLL" || strAppType == "WIN")
				LinkTool.SubSystem = subSystemWindows;
		}
	}
	catch(e)
	{
		throw e;
	}
}

