/*****************************************************************************
* This is the main code of a DeleD export plugin template for C++.          *
* Author: Salvatore Russo "Il Buzzo"                                        *
* Version: 2.0.0                                                            *
* DeleD version: 1.6+                                                       *
* License: GNU General Public License v3.                                      *
* Made with: Visual C++ 2005 Express Edition                                * 
* Additional Compiler Parameters: BUILD_DLL=1                               *
* End Notes: This is an adaptation to new plugin system, so there are some  *
*            changes from the old code, but as result I'm still adding to   *
*            code try{}catch() blocks for a better error handling.          *
*            It's not so difficult so if you don't want to wait for another *
*            version you can implement it without difficulty.               *
*****************************************************************************/


//Necessary for DLL to work
#include <windows.h>

//support.h contains some classes used, defined by me
//like String and StringList and some defined for DeleD
//in Jeroen sample code.
//Look at that file for major details.
#include "support.h"

#ifdef BUILD_DLL
#define DLL_EXPORT __declspec(dllexport)
#else
#define DLL_EXPORT 
#endif

//remember you need to have Microsoft platform sdk to link these libraries (free download)
#pragma comment(lib,"comdlg32.lib") //link to this to use openfilename() function of windows api;

#pragma comment(lib,"user32.lib") //link to user32.dll to use messagebox() function

//TCallBack variable used to receive a pointer by DeleD
TCallBack callback;

/* EXPORTED FUNCTIONS NECESSARY TO DELED

PCHAR PluginName() : returns name of Plugin
PCHAR PluginDescription() : returns a small description of Plugin
PCHAR PluginDeleDVersion() : returns version of DeleD
PCHAR PluginVersion() : returns version of Plugin
PCHAR PluginAuthor() : returns Author's name
PCHAR PluginEmail() : returns Author's e-mail
void PluginSetCallback(void*cp) : pass callback pointer from DeleD to callback
void PluginExecute() : it executes plugin code

notes:
PCHAR is the same as char* so it's a null terminated c string.
Each function is defined as DLLEXPORT for major details see
the DLLEXPORT definition in dll.h file.
As you can notice the function PluginType() is no longer used.
*/
extern "C" {
	/** returns name of Plugin*/
	PCHAR DLL_EXPORT __stdcall PluginName(){
		return (PCHAR)"DeleD [!output PROJECT_NAME] Plugin";
	}
	/** returns a small description of Plugin*/
	PCHAR DLL_EXPORT __stdcall PluginDescription(){
		return (PCHAR)"This is a plugin for DeleD generated with Visual Studio 2005 wizard.";
	}
	/**returns version of DeleD*/
	PCHAR DLL_EXPORT __stdcall PluginDeleDVersion(){
		return (PCHAR)"1.6+";
	}
	/**returns version of Plugin*/
	PCHAR DLL_EXPORT __stdcall PluginVersion(){
		return (PCHAR)"1.0";
	}
	/**returns Author's name*/
	PCHAR DLL_EXPORT __stdcall PluginAuthor(){
		return (PCHAR)"Your Name";
	}
	/**returns Author's e-mail*/
	PCHAR DLL_EXPORT __stdcall PluginEmail(){
		return (PCHAR)"Your e-mail";
	}
	/**pass callback pointer from DeleD to callback*/
	void DLL_EXPORT __stdcall PluginSetCallback(void *cp){
		callback=TCallBack(cp);
	}
	/**This executes plugin code*/
	void DLL_EXPORT __stdcall PluginExecute(){
		[!if EMPTY_PROJECT]
		//HERE YOU SHOULD ADD YOUR METHODS TO GET
		//SOMETHING FROM DELED.
		//IF YOU WANT TO KNOW HOW PLEASE
		//GIVE A LOOK TO REFERENCE
		//OR TRY WITHOUT CHECKING EMPTY PROJECT
		[!else]
		[!if CLASS_REFERENCES]
		//create a structure for Save Dialog (salva is the italian word for save :-))
		OPENFILENAME salva;

		//create a buffer for file name and path I hope it's big enough :-)
		char szFile[512];

		//inizialize structures to zeros
		ZeroMemory(&salva, sizeof(OPENFILENAME));
		ZeroMemory(szFile, 512*sizeof(char));

		//initialization of structure salva
		salva.lStructSize = sizeof(OPENFILENAME);
		salva.hwndOwner = NULL;//Parent window
		salva.lpstrFile = szFile;//file name buffer
		salva.nMaxFile = 512;//buffer size
		salva.lpstrFilter = "DXS Files(*.dxs)\0*.dxs\0Text files(*.txt)\0*.txt\0All files(*.*)\0*.*\0";//filter data
		salva.nFilterIndex = 1;//default index
		salva.lpstrFileTitle = NULL;
		salva.lpstrTitle = "Select a file to export data string from DeleD";//Save dialog title
		salva.lpstrInitialDir = NULL;
		salva.Flags = OFN_OVERWRITEPROMPT;//ask to overwrite an existent file

		//Create a scene
		Scene *scene=new Scene();

		//get all materials and copy to scene
		GetMaterials(false,false,scene);

		//get all objects
		GetPrimitives(false,false,scene);

		//get all light objects
		GetLights(false,false,scene);

		//now let's add some info that can't be obtained by DeleD plugin system
		//scene name
		scene->name=String("MyDeleDScene");
		//scene ambient
		scene->ambient=Color(255,255,255);
		//scene shadow color
		scene->shadow=75;
		//Let's get file name
		if(GetSaveFileName(&salva)) {
			FILE *F1;
			F1=fopen (salva.lpstrFile, "w");//creates a new file
			//write basic infos
			fprintf(F1,"<scene version=\"1.6\">\n");
			fprintf(F1,"%s\n%s\n%s\n%s\n</scene>",scene->to_deled_settings(),scene->to_deled_materials(),
				scene->to_deled_primitives(),scene->to_deled_lights());
			fclose(F1);
		}

		MessageBox(NULL,"DeleD C++ templates version 2.0 written by:\n Salvatore Russo.\nLot of thanks to Jeroen and Paul Jan\nfor this new improved version of DeleD!!!","Information",0);
		[!else]
		//create a structure for Save Dialog (salva is the italian word for save :-))
		OPENFILENAME salva;

		//create a buffer for file name and path I hope it's big enough :-)
		char szFile[512];

		//inizialize structures to zeros
		ZeroMemory(&salva, sizeof(OPENFILENAME));
		ZeroMemory(szFile, 512*sizeof(char));

		//initialization of structure salva
		salva.lStructSize = sizeof(OPENFILENAME);
		salva.hwndOwner = NULL;//Parent window
		salva.lpstrFile = szFile;//file name buffer
		salva.nMaxFile = 512;//buffer size
		salva.lpstrFilter = "DXS Files(*.dxs)\0*.dxs\0Text files(*.txt)\0*.txt\0All files(*.*)\0*.*\0";//filter data
		salva.nFilterIndex = 1;//default index
		salva.lpstrFileTitle = NULL;
		salva.lpstrTitle = "Select a file to export data string from DeleD";//Save dialog title
		salva.lpstrInitialDir = NULL;
		salva.Flags = OFN_OVERWRITEPROMPT;//ask to overwrite an existent file


		if(GetSaveFileName(&salva))
		{
			FILE *F1;
			F1=fopen (salva.lpstrFile, "w");//creates a new file
			//write basic infos
			fprintf(F1,"<scene version=\"1.6\">\n  <settings name=\"new scene\" author=\"\" comments=\"\" shadowOpacity=\"75\">\n");
			fprintf(F1,"    <ambient r=\"255\" g=\"255\" b=\"255\" />\n  </settings>");
			fprintf(F1,"%s",GetMaterials(false, false).c_str());
			fprintf(F1,"%s",GetPrimitives(false, false).c_str());
			fprintf(F1,"%s",GetLights(false, false).c_str());
			fprintf(F1,"</scene>");
			fclose(F1);

		}
		MessageBox(NULL,"DeleD C++ templates version 2.0 written by:\n Salvatore Russo.\nLot of thanks to Jeroen and Paul Jan\nfor this new improved version of DeleD!!!","Information",0);

		[!endif]
		[!endif]

	}
}//END EXTERN C
