//This support library has been made by Salvatore Russo and is released under GNU public license.
//It uses standard template libraries to create String class and StringList class used in DeleD
//plugins made by me.
//It also includes some facilities to add or retrieve data, or convert a long, integer, bool or unsigned to a string,
//and handle callback errors.

#ifndef _SUPPORT_H_
#define _SUPPORT_H_

//standard classes to use
#include <cstdio>
#include <string>
#include <deque>
[!if CLASS_REFERENCES]
#include <vector>
#include <math.h>
[!endif]

//so you know error types
#include "[!output PROJECT_NAME].h"


typedef std::string stdstring;

/**Define a class String based on STL string.*/
class String : public stdstring {
public:
	/**basic constructor.*/
	String() : stdstring() {};
	/**basic constructor.*/
	String(const char* str) : stdstring(str) {};
	/**basic copy constructor.*/
	String(const stdstring& str) : stdstring(str) {};
	/**trim function in my implementation removes spaces but also \\t and \\n chars
	*from end and beginning of string only.*/
	void Trim(){
		int k = (this->length())-1;
		while(k>=0){
			if((*this)[k]==' ' || (*this)[k]=='\t' || (*this)[k]=='\n'){
				this->erase(k,1);//trim
				k=k-1;
			}
			else k=-1;
		}
		k=0;
		while(k < this->length()){
			if((*this)[k]==' ' || (*this)[k]=='\t' || (*this)[k]=='\n'){
				this->erase(k,1);//trim
			}
			else k=this->length();
		}
	}
	/**replace all instaces of "from" char with "to" char.
	*\param from Char to eliminate.
	*\param to Char to use in substitution.*/
	void Replace(char from,char to){
		int k = (this->length())-1;
		while(k>=0){
			if((*this)[k]==from) (*this)[k]=to;//replace
			k=k-1;
		}
	}
	/**Get a substring of this string.
	*\param ind Starting index of substring.
	*\param len Length of substring.
	*\return Wanted string.*/
	String subString(int ind,int len){return this->substr(ind,len);};
};


using std::deque;

/**Define a class StringList based on STL deque and the defined class String.*/
class StringList : public deque <String> {
public:
	/**Adds a String to StringList. \param str String to add.*/
	void Add(String str){push_back(str);}

	/**Loads a stringlist from a file
	*note that each String added to StringList
	*is separed by a \n character and it's present
	*at the end of line. \param filename Name of file to open to get strings.*/
	void LoadFromFile(const char* filename){
		FILE *nuovofile;
		nuovofile=fopen(filename,"r");//opens new file
		int c;
		String str;
		c = fgetc(nuovofile);
		while(c!=EOF){
			if(c!='\n'){
				str.append((char*)&c);
			}
			else {
				str.append((char*)&c);
				str.Trim();
				Add(str);
				str.clear();
			};
			c = fgetc (nuovofile);


		}
		fclose(nuovofile);
	}

	/**Returns a String containing all the text included in
	*the StringList. \return A String containing all text.*/
	String GetText(){
		String str;
		for(int i=0;i<this->size();i++){
			str=str+(*this)[i];
		}
		return str;
	}

	/**Returns a String containing all the text included in
	*the StringList with a newline for each string in list. \return A String containing all text.*/
	String GetTextNewLine(){
		String str;
		for(int i=0;i<this->size();i++){
			str=str+(*this)[i]+String("\n");
		}
		return str;
	}
};

[!if CLASS_REFERENCES]

/**Define a namespace to represent DeleD layer types.*/
namespace LAYER_TYPE
{
	/**texture layer.*/
	const String TEXTURE=String("texture");
	/**lightmap layer.*/
	const String LIGHTMAP=String("lightmap");
	/**color layer.*/
	const String COLOR=String("color");
}

/**Define a namespace to represent DeleD snap types.*/
namespace SNAP_TYPE
{
	/**no snapping.*/
	const String NONE=String("none");
	/**vertex snapping*/
	const String VERTEX=String("vertex");
	/**center snapping*/
	const String CENTER=String("center");
}

/**Define a namespace to represent DeleD blend types.*/
namespace BLEND_TYPE
{
	/**inactive layer.*/
	const String INACTIVE=String("inactive");
	/**add layer.*/
	const String ADD=String("add");
	/**replace layer.*/
	const String REPLACE=String("replace");
	/**alphablend layer.*/
	const String ALPHABLEND=String("alphablend");
	/**modulate layer.*/
	const String MODULATE=String("modulate");
}

/**Define a namespace to represent DeleD light types.*/
namespace LIGHT_TYPE
{
	/**spot light.*/
	const String SPOT=String("spot");
	/**directional light.*/
	const String DIRECTIONAL=String("directional");
	/**point light.*/
	const String POINT=String("point");
}

/**Define a namespace to represent DeleD primitives.*/
namespace PRIMITIVE_TYPE
{
	/**grid primitive.*/
	const String GRID=String("grid");
	/**polyline primitive.*/
	const String POLYLINE=String("polyline");
	/**rectangle primitive.*/
	const String RECTANGLE=String("rectangle");
	/**cube primitive.*/
	const String CUBE=String("cube");
	/**ellipse primitive.*/
	const String ELLIPSE=String("ellipse");
	/**pyramid primitive.*/
	const String PYRAMID=String("pyramid");
	/**cylinder primitive.*/
	const String CYLINDER=String("cylinder");
	/**sphere primitive.*/
	const String SPHERE=String("sphere");
	/**torus primitive.*/
	const String TORUS=String("torus");
	/**text primitive.*/
	const String TEXT=String("text");
	/**userdefined primitive.*/
	const String USERDEF=String("userdefined");
}
[!endif]

/*!Convert long integer to a string.
*\param x long value to convert.
*\return Value converted to String.*/
String toString(long x) ;

/*!Convert bool to a string.
*\param x bool value to convert.
*\return Value converted to String.*/
String toString(bool x) ;

/*!Convert integer to a string.
*\param x int value to convert.
*\return Value converted to String.*/
String toString(int x) ;

/*!Convert unsigned integer to a string.
*\param x unsigned value to convert.
*\return Value converted to String.*/
String toString(unsigned x) ;

/*!Convert double to a string.
*\param x double value to convert.
*\return Value converted to String.*/
String toString(double x) ;

/*!Convert float to a string.
*\param x float value to convert.
*\return Value converted to String.*/
String toString(float x) ;

/*!This function subdivides a string in a list of strings.
*\param str Data to parse.
*\param divider Delimiter to use.
*\return StringList after parsing.*/
StringList SubdivideString(String str,String divider);


#ifndef NO_MESSAGES
/*!This function show an error message when giving callback error code.
*\param error Error code returned by DeleD. */
void ErrorMessage(unsigned error);
#endif


[!if CLASS_REFERENCES]
/**Color class is a basic color representation. This class is used to define a standard RGBA color, and for the moment is used by SpotLight, DirectionalLight and PointLight classes to define diffuse and specular colors.
\todo add some other helper functions.
*Sample Usage:
\verbatim
--create a red color
Color mycol(255,0,0,0);
...
\endverbatim
*/
class Color{
public:
	/**Red amount. Values have to be comprised between 0 and 255.*/
	unsigned r;
	/**Green amount. Values have to be comprised between 0 and 255.*/
	unsigned g;
	/**Blue amount. Values have to be comprised between 0 and 255.*/
	unsigned b;
	/**Alpha amount. Values have to be comprised between 0 and 255.*/
	unsigned a;
	/**Basic constructor. Creates a black color.*/
	Color(){r=0;g=0;b=0;a=0;};
	/**Create a specified RGB color. \param red Red amount of wanted color. \param green Green amount of wanted color. \param blue Blue amount of wanted color. \param alpha Alpha amount of wanted color.*/
	Color(unsigned red,unsigned green, unsigned blue,unsigned alpha=255) {r=red;g=green;b=blue;a=alpha;};
	/**Basic equality operator. \return true if the two colors are the same false in other cases.*/
	bool operator==(const Color &other) {return (r==other.r && g==other.g && b==other.b && a==other.a);};
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Color string representation. \return A string representation of Color for use in Deled files and internal callbacks.*/
	const char* to_deled()
	{
		deled=String("<color r=\"")+toString(r)+String("\" g=\"")+toString(g)+
			String("\" b=\"")+toString(b)+String("\" a=\"")+toString(a)+String("\" />");
		return deled.c_str();
	};

};


/** Layer class is a representation of DeleD layers. It represents a single layer contained
* in a single material. Note that a material can contain other layers up to four (one of which is lightmap layer).
* Sample usage:
\verbatim
//create a new layer
Layer mylayer=new Layer();
mylayer->type=LAYER_TYPE::TEXTURE;
...
\endverbatim
*/
class Layer{
public:
	/**Deled Layer type. DeleD Layer type admitted values are: LAYER_TYPE::TEXTURE texture layers, LAYER_TYPE::LIGHTMAP for lightmap layers and LAYER_TYPE::COLOR for color layers.*/
	String type;
	/**Deled Layer blend mode. DeleD Layer blending mode admitted values are: BLEND_TYPE::INACTIVE inactive layers, BLEND_TYPE::ADD for adding layers, BLEND_TYPE::REPLACE to replace layers,BLEND_TYPE::MODULATE to modulate layers and BLEND_TYPE::ALPHABLEND for transparent layers.*/
	String blend; 
	/**Get file string. \return this DeleD layer file name, please refer to file property to know more.*/
	const char* get_file(){return file.c_str();};
	/**Set file string. \param filename DeleD layer file name, please refer to file property to know more.*/
	void set_file(char *filename){file=String(filename);};
	/**Layer string representation. \return this function returns a string representation of layer for use in Deled files and internal callbacks.*/
	const char* to_deled(){
		if(type!=LAYER_TYPE::COLOR)
			deled=String("<layer type=\"")+type+String("\" blend=\"")+blend+String("\">\n")+
			String("<texture file=\"")+file+String("\" />\n</layer>");
		else
			deled=String("<layer type=\"")+type+String("\" blend=\"")+blend+String("\">\n")+
			String(color.to_deled())+String("\n</layer>");
		return deled.c_str();
	};
	/**Default Layer constructor. Doesn't do much at the moment.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	Layer(){type.clear();file.clear();deled.clear();};
	/**DeleD layer file name. In case type is TEXTURE_TYPE::TEXTURE or TEXTURE_TYPE::LIGHTMAP it's a subpath (Floors\\Floor16.jpg)*/
	String file;
	/**DeleD layer color. In case type is TEXTURE_TYPE::COLOR it's defined and is a color*/
	Color color;
	/**Internal property to hold strings in DeleD format.*/
	String deled;
};


/** Material class is a basic representation of DeleD materials. It's strictly connected to Layer class
*because it can contain up to four layers, with some other properties.
* Sample usage:
\verbatim
//create a new material
Material *mymaterial=new Material();
mymaterial->name=String("system");
//or mymaterial->set_name("system");
mymaterial->addLayer(mylayer);
...
\endverbatim
*/
class Material{
public:
	/**This material id in DeleD. Note that materials id are unique*/
	int id;
	/**Name of this material (i.e.\ "system"). You can get or set accessing it.*/
	String name;
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Internal property to hold a tag.*/
	String tag;
	/**Internal property to add comments.*/
	String comments;
	/**Tells if this material is used. You can get or set accessing it.*/
	bool used;
	/**Tells if this material can be lightmapped. You can get or set accessing it.*/
	bool lightmap;
	/**Tells if this material can cast shadows. You can get or set accessing it.*/
	bool castShadows;
	/**Tells if this material can receive shadows. You can get or set accessing it.*/
	bool receiveShadows;
	/**Vector of layers that are contained in this material. Note that you can use all standard vector properties.*/
	std::vector<Layer> layers;
	/**Get name property. \return this name.*/
	const char* get_name(){return name.c_str();};
	/**Set name property. \param mat_name this name.*/
	void set_name(char *mat_name){name=String(mat_name);};
	/**Add a Layer to current material. \param layer Layer to add to current layer list of this material.*/
	void addLayer(Layer layer){layers.push_back(layer);};
	/**Remove a Layer from current material. \param index index of layer to remove*/
	void removeLayer(int index){std::vector<Layer>::iterator p=layers.begin(); if(index<layers.size() && index>-1){p+=index;layers.erase(p);}};
	/**Get Layer wanted. You can get desidered Layer (i.e. to modify) accessing with its index \param index index of Layer to get \return returns wanted layer. */
	Layer *getLayer(int index){if(index>=0 && index<layers.size()) return &layers[index]; else return NULL;};
	/**Get number of layers in this material. \return returns Layers number in this material.*/
	int getSize(){return layers.size();};
	/**Material string representation. \return this function returns a string representation of material for use in Deled files and internal callbacks.*/
	const char* to_deled(){
		deled=String("<material id=\"")+toString(id)+String("\" name=\"")+name+String("\" used=\"")+toString(used)+
			String("\" lightmap=\"")+toString(lightmap)+String("\" castShadows=\"")+toString(castShadows)+String("\" receiveShadows=\"")+toString(receiveShadows)+String("\">\n");
		if(tag!=String(""))
			deled=deled+String("<tag>")+tag+String("</tag>\n");
		if(comments!=String(""))
			deled=deled+String("<comments>")+comments+String("</comments>\n");
		for(int i=0;i<layers.size();i++)
			deled=deled+String(layers[i].to_deled())+String("\n");
		deled=deled+String("</material>");
		return deled.c_str();
	};
	/**Default Material constructor. Doesn't do much at the moment.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	Material(){name.clear();layers.clear();deled.clear();tag.clear();comments.clear();};
};



/** MaterialCategory class is a basic representation of DeleD category materials that contains materials. It's strictly connected to Material and Layer classes.
* Sample usage:
\verbatim
//create a new category
MaterialCategory *mymaterialcategory=new MaterialCategory();
mymaterialcategory->name=String("System");
//or mymaterialcategory->set_name("System");
mymaterialcategory->addMaterial(mymaterial);
...
\endverbatim
*/
class MaterialCategory{
public:
	/**Name of this material category (i.e.\ "System"). You can get or set accessing it.*/
	String name;
	/**Vector of materials that are contained in this category. Note that you can use all standard vector properties.*/
	std::vector<Material> materials;
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Get name property. \return this name.*/
	const char* get_name(){return name.c_str();};
	/**Set name property. \param cat_name name given to this category.*/
	void set_name(char *cat_name){name=String(cat_name);};
	/**Add a Material to current CategoryMaterial. \param material Material to add to current material list of this material category.*/
	void addMaterial(Material material){materials.push_back(material);};
	/**Remove a Material from current CategoryMaterial. \param index index of material to remove*/
	void removeMaterial(int index){std::vector<Material>::iterator p=materials.begin(); if(index<materials.size() && index>-1){p+=index;materials.erase(p);}};
	/**Remove a Material from current CategoryMaterial. \param id id of material to remove*/
	void removeMaterialFromId(int id){removeMaterial(getIndex(id));};
	/**Remove a Material from current CategoryMaterial. \param mat_name Name of material to remove, only first instance removed.*/
	void removeMaterial(char *mat_name){std::vector<Material>::iterator p=materials.begin();int index=getIndex(mat_name); if(index<materials.size() && index>-1){p+=index;materials.erase(p);}};
	/**Get Material wanted. You can get desidered Material (i.e. to modify) accessing with its name \param mat_name name of Material to get \return returns wanted material. */
	Material *getMaterial(char *mat_name){int index=getIndex(mat_name); if(index>=0) return &materials[index]; else return NULL;};
	/**Get Material wanted. You can get desidered Material (i.e. to modify) accessing with its index \param index index of Material to get \return returns wanted material. */
	Material *getMaterial(int index){if(index>=0 && index<materials.size()) return &materials[index]; else return NULL;};
	/**Get Material wanted using its id. You can get desidered Material (i.e. to modify) accessing with its index \param id id of Material to get \return returns wanted material. */
	Material *getMaterialFromId(int id){return getMaterial(getIndex(id));};

	/**Get index relative to a Material with the specified name. \param mat_name Name of the material to obtain index \return returns index for the wanted material. */
	int getIndex(char *mat_name){
		int index=-1;
		for(int i=0;i<materials.size();i++){
			if(materials[i].name==String(mat_name)) {
				index=i;
				break;
			}
		}
		return index;
	};
	/**Get index relative to a Material with the specified id. \param id id of the material to obtain index \return returns index for the wanted material. */
	int getIndex(int id){
		int index=-1;
		for(int i=0;i<materials.size();i++){
			if(materials[i].id==id) {
				index=i;
				break;
			}
		}
		return index;
	};
	/**Get number of materials in this material category. \return returns materials number in this material category.*/
	int getSize(){return materials.size();};

	/**MaterialCategory string representation. \return this function returns a string representation of material category for use in Deled files and internal callbacks.*/
	const char* to_deled(){
		deled=String("<category name=\"")+name+String("\">\n");
		for(int k=0;k<materials.size();k++)
			deled=deled+String(materials[k].to_deled())+String("\n");
		deled=deled+String("</category>");
		return deled.c_str();
	};
	/**Default Material Category constructor. Doesn't do much at the moment.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	MaterialCategory(){
		name.clear();
		materials.clear();
		deled.clear();
	};
	/**Material Category constructor. \param cat_name initialize a MaterialCategory object with cat_name name.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	MaterialCategory(String cat_name){
		name=cat_name;
		materials.clear();
		deled.clear();
	};
};

/** Basic Vertex class that represents a point in 3D space. This class is used
* to define vertices in Object class as well as direction in lights classes etc...
*Sample Usage:\todo add other helper functions.
\verbatim
//create a new vertex
Vertex myvert();
Vertex myvert1=myvert*2;
...
\endverbatim
*/
class Vertex{
public:
	/** Coordinates of this vertex. Basic coordinates of this vertex.*/
	double x,y,z;
	/**Standard constructor. Doesn't do much at the moment.*/
	Vertex(){};
	/**Create a new vertex initializing its coords. \param X x coord\param Y y coord\param Z z coord.*/
	Vertex(double X,double Y, double Z){x=X;y=Y;z=Z;};
	/**Sum operator. It simply sums two vertices like v3=v1+v2, operation is done for ordered components.*/
	Vertex operator+(const Vertex& other){return Vertex(x+other.x,y+other.y,z+other.z);};
	/**Subtract operator. It simply subtracts two vertices like v3=v1-v2, operation is done for ordered components*/
	Vertex operator-(const Vertex& other){return Vertex(x-other.x,y-other.y,z-other.z);};
	/**Divide operator. It simply divides each vertex component for the relative one of the other vertex.*/
	Vertex operator/(const Vertex& other){return Vertex(x/other.x,y/other.y,z/other.z);};
	/**Divide operator. Divides a vector for a scalar.*/
	Vertex operator/(const double& other){return Vertex(x/other,y/other,z/other);};
	/**Multiply operator. It simply multiplies each vertex component for the relative one of the other vertex.*/
	Vertex operator*(const Vertex& other){return Vertex(x*other.x,y*other.y,z*other.z);};
	/**Multiply operator. multiplies a vector for a scalar.*/
	Vertex operator*(const double& other){return Vertex(x*other,y*other,z*other);};
	/**Retuns true if two vertices have the same coords.*/
	bool operator==(const Vertex& other){return (x==other.x && y==other.y && z==other.z);};
	/**Evaluate the euclidean distance between this and other vertex. \param other Other vertex for evaluate distance \return Euclidean distance.*/
	double distance(const Vertex other){Vertex tmp=(*this)-other;tmp=tmp*tmp;return sqrt(tmp.x+tmp.y+tmp.z);};
	/**Evaluate the squared euclidean distance between this and other vertex. \param other Other vertex for evaluate distance \return Squared euclidean distance, this can be useful cause you don't evaluate square root (less calculations).*/
	double squaredistance(const Vertex other){Vertex tmp=(*this)-other;tmp=tmp*tmp;return (tmp.x+tmp.y+tmp.z);};
};

/** Primitive Vertex class that represents a point in 3D space with id to be used in DeleD primitive.
*Sample Usage:\todo add other helper functions.
\verbatim
//create a new primitive vertex having id=0
PrimitiveVertex myvert(0,myvert);
...
\endverbatim
*/
class PrimitiveVertex
{
public:
	/**id of this vertex*/
	int id;
	/**Vertex itself.*/
	Vertex v;
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Standard constructor. Doesn't do much at the moment.*/
	PrimitiveVertex(){};
	/**Standard constructor. \param ID id of this vertex \param vert vertex definition.*/
	PrimitiveVertex(int ID,Vertex vert){id=ID;v=vert;};
	/**PrimitiveVertex string representation. \return A string representation of PrimitiveVertex for use in Deled files and internal callbacks.*/
	const char* to_deled(){deled=String("<vertex id=\"")+toString(id)+String("\" x=\"")+toString(v.x)+String("\" y=\"")+toString(v.y)+String("\" z=\"")+toString(v.z)+String("\" />");return deled.c_str();}
};


/** Basic UV coordinates representation. This class is largely used in PolyVertex class, or can be used
* for other scopes like 2D points.
*Sample Usage:\todo add other helper functions.
\verbatim
--create a new uv coord
UVCoords myuv(1.0,1.0);
myuv=myuv*2;
...
\endverbatim
*/
class UVCoords{
public:
	/**Basic Coordinates values. u and v values of this UVCoords.*/
	double u,v;
	/**Standard constructor. Doesn't do much.*/
	UVCoords(){};
	/**Create a new UVCoords initializing u and v values.\param U u value of the new UVCoord\param V v value of the new UVCoord.*/
	UVCoords(double U, double V){u=U;v=V;};
	/**Sum operator. It simply sums two uvcoords like u3=u1+u2, operation is done for ordered components.*/
	UVCoords operator+(const UVCoords& other){return UVCoords(u+other.u,v+other.v);};
	/**Subtract operator. It simply subtracts two uvcoords like u3=u1-u2, operation is done for ordered components.*/
	UVCoords operator-(const UVCoords& other){return UVCoords(u-other.u,v-other.v);};
	/**Divide operator. It simply divides each uvcoord component for the relative one of the other uvcomponent.*/
	UVCoords operator/(const UVCoords& other){return UVCoords(u/other.u,v/other.v);};
	/**Divide operator. Divides a uvcoord for a scalar.*/
	UVCoords operator/(const double& other){return UVCoords(u/other,v/other);};
	/**Multiply operator. It simply multiplies each uvcoord component for the relative one of the other uvcoord.*/
	UVCoords operator*(const UVCoords& other){return UVCoords(u*other.u,v*other.v);};
	/**Multiply operator. multiplies a uvcoord for a scalar.*/
	UVCoords operator*(const double& other){return UVCoords(u*other,v*other);};
	/**Retuns true if two uvcoords have the same coords.*/
	bool operator==(const UVCoords& other){return (u==other.u && v==other.v);};
	/**Evaluate the euclidean distance between this and other uvcoord. \param other Other uvcoord for evaluate distance \return Euclidean distance.*/
	double distance(const UVCoords other){UVCoords tmp=(*this)-other;tmp=tmp*tmp;return sqrt(tmp.u+tmp.v);};
	/**Evaluate the squared euclidean distance between this and other uvcoord. \param other Other uvcoord for evaluate distance \return Squared euclidean distance, this can be useful cause you don't evaluate square root (less calculations).*/
	double squaredistance(const UVCoords other){UVCoords tmp=(*this)-other;tmp=tmp*tmp;return (tmp.u+tmp.v);};

};

/** PolyVertex class that represents a vertex definition to be used in DeleD poly.
*Sample Usage:\todo add other helper functions.
\verbatim
//create a new PolyVertex having id=0 and one set of UV coords
PolyVertex myvert(0,UVCoords(0.0,1.0));
...
\endverbatim
*/
class PolyVertex{
public:
	/**id of this vertex*/
	int vertex_id;
	/**Vector of UVCoords*/
	std::vector<UVCoords> vertex_uv;
	/**Adds an UVCoords to actual PolyVertex. \param uv UVCoords to add to this poly vertex.*/
	void addUV(UVCoords uv){vertex_uv.push_back(uv);};
	/**Remove an UVCoords to actual PolyVertex. \param index Position of UVCoords to remove in this poly.*/
	void removeUV(int index){std::vector<UVCoords>::iterator p=vertex_uv.begin(); if(index<vertex_uv.size() && index>-1){p+=index;vertex_uv.erase(p);}};
	/**Get uvcoords list size. \return Number of items in uvcoords list of this poly.*/
	int getUVSize(){return vertex_uv.size();};
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Standard constructor. Doesn't do much at the moment.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	PolyVertex(){vertex_uv.clear();}
	/**Standard constructor. \param id ID of this PolyVertex.
	*/
	PolyVertex(int id){vertex_id=id;vertex_uv.clear();}
	/**Standard constructor. \param id ID of this PolyVertex. \param UV first set of uv coords.
	*/
	PolyVertex(int id, UVCoords UV){vertex_id=id;vertex_uv.clear();vertex_uv.push_back(UV);}
	/**Standard constructor. \param id ID of this PolyVertex. \param UV a vector containing uv coords.
	*/
	PolyVertex(int id, std::vector<UVCoords> UV){vertex_id=id;vertex_uv.clear();for(int i=0;i<UV.size();i++)vertex_uv.push_back(UV[i]);}
	/**PolyVertex string representation. \return A string representation of PolyVertex for use in Deled files and internal callbacks.*/
	const char *to_deled(){
		deled=String("<vertex vid=\"")+toString(vertex_id)+String("\" ");
		for(int i=0;i<vertex_uv.size();i++)
			deled=deled+String("u")+toString(i)+String("=\"")+toString(vertex_uv[i].u)+String("\" v")+toString(i)+String("=\"")+toString(vertex_uv[i].v)+String("\" ");
		deled=deled+String("/>");
		return deled.c_str();};
};

/** Poly class is a basic representation of DeleD polys. It's strictly connected to PolyVertex class
* and is largely used in Primitive class.
*Sample Usage:
\verbatim
--create a new poly
Poly *mypoly=new Poly();
mypoly->addVertex(PolyVertex(0,UVCoords(0.0,1.0));
mypoly->addVertex(PolyVertex(2,UVCoords(0.0,1.0));
mypoly->addVertex(PolyVertex(3,UVCoords(0.0,1.0));
...
\endverbatim
*/
class Poly{
public:
	/**Material id. This is Material id to use for this poly. Note that a material with such id must exist.*/
	int mat;
	/**A vector containing all PolyVertex . Note that you can use all standard vector methods.*/
	std::vector<PolyVertex> poly_vertices;
	/**Adds a PolyVertex to this poly. A poly is made of some vertices but you must just set an int that represents
	*vertex position. In fact in DeleD, for each Object there is a list of vertices, you must just add the index of vertex (note that
	*it's a zero-based list, so index will start from 0).\param vertex Item to add to current PolyVertex indices list.*/
	void addVertex(PolyVertex vertex){poly_vertices.push_back(vertex);};
	/**Remove from current list of vertices indexes the element at "index" position.\param index Position of item to remove in current vertex indices list.*/
	void removeVertex(int index){std::vector<PolyVertex>::iterator p=poly_vertices.begin(); if(index<poly_vertices.size() && index>-1){p+=index;poly_vertices.erase(p);}};
	/**Remove from current list of vertices indexes the element having the wanted id.\param id ID of PolyVertex to remove in current vertex indices list.*/
	void removeVertexFromId(int id){removeVertex(getIndex(id));};
	/**Get index of PolyVertex with current id. \param id ID of PolyVertex. \return index of PolyVertex with the wanted ID.*/
	int getIndex(int id){
		int index=-1;
		for(int i=0;i<poly_vertices.size();i++){
			if(poly_vertices[i].vertex_id==id) {
				index=i;
				break;
			}
		}
		return index;
	};
	/**Get PolyVertex list size. \return Number of items in PolyVertex list.*/
	int getVertexSize(){return poly_vertices.size();};
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Poly string representation. \return A string representation of Poly for use in Deled files and internal callbacks.*/
	const char *to_deled(){
		deled=String("<poly mid=\"")+toString(mat)+String("\">\n");
		for(int i=0;i<poly_vertices.size();i++)
			deled=deled+String(poly_vertices[i].to_deled())+String("\n");
		deled=deled+String("</poly>");
		return deled.c_str();
	};
	/**Standard constructor. Doesn't do much at the moment.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	Poly(){poly_vertices.clear();};
	/**Standard constructor.\param material sets the material used by this poly.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	Poly(int material){mat=material;poly_vertices.clear();};
};


/**Primitive class is a basic representation of a DeleD primitive. Because of its structure it's strictly connected
* to PrimitiveVertex and PolyVertex classes.
*Sample Usage:
\verbatim
--create a new primitive
Primitive *myobj=new Primitive();
myobj->addVertex(PrimitiveVertex(0,Vertex(0,0,0)));
myobj->addVertex(PrimitiveVertex(1,Vertex(-128,0,0)));
myobj->addVertex(PrimitiveVertex(2,Vertex(-128,0,-128)));
myobj->addVertex(PrimitiveVertex(3,Vertex(0,0,-128)));
...
\endverbatim
*/
class Primitive{
public:
	/**Name of this primitive. Name of this Primitive that can be get and set.*/
	String name;
	/**Get name property. \return name of this primitive.*/
	const char * get_name(){return name.c_str();};
	/**Set name property. \param newname name of this primitive.*/
	void set_name(char *newname){name=String(newname);};
	/**Id of this Primitive. This is unique id of this Primitive.*/
	int id;
	/**Property to hold comments.*/
	String comments;
	/**Type of this Object. Admitted types are : \li \c PRIMITIVE_TYPE::GRID for a grid.\li \c PRIMITIVE_TYPE::POLYLINE for a polyline.\li \c PRIMITIVE_TYPE::RECTANGLE for a rectangle.
	*\li \c PRIMITIVE_TYPE::CUBE for a cube.\li \c PRIMITIVE_TYPE::ELLIPSE for an ellipse.\li \c PRIMITIVE_TYPE::PYRAMID for a pyramid.\li \c PRIMITIVE_TYPE::CYLINDER for a cylinder.
	*\li \c PRIMITIVE_TYPE::SPHERE for a sphere. \li \c PRIMITIVE_TYPE::TORUS for a torus. \li \c PRIMITIVE_TYPE::TEXT for text. \li \c PRIMITIVE_TYPE::USERDEF for user defined shape.*/
	String type;
	/**Visibility of this Object. Hide or show this Object.*/
	bool visible;
	/**Snap method of this Object (aligning to grid). Admitted values are :  \li \c SNAP_TYPE::NONE no snapmethod.\li \b SNAP_TYPE::VERTEX snap vertices.\li \c SNAP_TYPE::CENTER snap center of Object.*/
	String snapmethod;
	/**Group id of this Object. For now set this to \c -1.*/
	int groupid;
	/**Auto UV coords for this Object.*/
	bool autoUV;
	/**User defined info for this Object. A string representing user infos for this object.*/
	String tag;
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Get tag property.\return tag of this object.*/
	const char * get_tag(){return tag.c_str();};
	/**Get tag property.\param newtag tag of this object.*/
	void set_tag(char *newtag){tag=String(newtag);};
	/**Get comments property.\return comments of this object.*/
	const char * get_comments(){return comments.c_str();};
	/**Get comments property.\param newcomm set a new comment for this object.*/
	void set_comments(char *newcomm){tag=String(newcomm);};
	/**A vector containing all PrimitiveVertex elements of this object. Note that you can use all standard vector methods.*/
	std::vector<PrimitiveVertex> vertices;
	/**Obtain number of vertices in this Object. \return Number of vertices contained in vertices list.*/
	int getVertexSize(){return vertices.size();};
	/**Adds a Vertex at the end of current vertices list.\param vert Vertex to be added at the end of the current vertices list.*/
	void addVertex(PrimitiveVertex vert){vertices.push_back(vert);};
	/**Remove Vertex at a particular position in list. \param index index of Vertex to remove from current vertices list. \note vertices list is
	* a zero based list, so position \b 0 will be the first one, and the last is \b num_vertices-1.*/
	void removeVertex(int index){{std::vector<PrimitiveVertex>::iterator p=vertices.begin(); if(index<vertices.size() && index>-1){p+=index;vertices.erase(p);}};};
	/**Get a Vertex in vertices list to modify it. \param index index of Vertex to obtain from current vertices list. \return Vertex wanted so you can modify it. \note If you get a Vertex
	* with this function then each modification will be automatically reported in Vertex in vertices list, cause you'll get just a pointer to an existent Vertex.*/
	PrimitiveVertex *getVertex(int index){return &vertices[index];};
	/**A vector containing all Poly elements of this object. Note that you can use all standard vector methods.*/
	std::vector<Poly> polygons;
	/**Obtain number of polys in this Object. \return Number of polys contained in polys list.*/
	int getPolySize(){return polygons.size();};
	/**Adds a Poly at the end of current polys list.\param polygon Poly to be added at the end of the current polys list.*/
	void addPoly(Poly polygon){polygons.push_back(polygon);};
	/**Remove Poly at a particular position in list. \param index index of Poly to remove from current polys list. \note polys list is
	* a zero based list, so position \c 0 will be the first one, and the last is \c num_polys-1.*/
	void removePoly(int index){std::vector<Poly>::iterator p=polygons.begin(); if(index<polygons.size() && index>-1){p+=index;polygons.erase(p);}};
	/**Get a Poly in polys list to modify it. \param index index of Poly to obtain from current polys list. \return Poly wanted so you can modify it. \note If you get a Poly
	* with this function then each modification will be automatically reported in Poly in polys list , cause you'll get just a pointer to an existent Poly.*/
	Poly *getPoly(int index){return &polygons[index];};
	/**Primitive string representation. \return A string representation of Object for use in Deled internal callbacks.\note This function is not suitable to be used for DeleD file writing cause it lacks new line characters, it's used just to pass data to DeleD internally with functions.*/
	const char* to_deled(){
		deled=String("<primitive id=\"")+toString(id)+String("\" name=\"")+name+String("\" type=\"")+type+String("\" visible=\"")+toString(visible)+
			String("\" snap=\"")+snapmethod+String("\" autoUV=\"")+toString(autoUV)+String("\" groupID=\"")+toString(groupid)+String("\">\n");
		if(tag!=String(""))
			deled=deled+String("<tag>")+tag+String("</tag>\n");
		if(comments!=String(""))
			deled=deled+String("<comments>")+comments+String("</comments>\n");
		deled=deled+String("<vertices>\n");
		for(int i=0;i<vertices.size();i++)
			deled=deled+String(vertices[i].to_deled())+String("\n");
		deled=deled+String("</vertices>\n<polygons>\n");
		for(int i=0;i<polygons.size();i++)
			deled=deled+String(polygons[i].to_deled())+String("\n");
		deled=deled+String("</polygons>\n</primitive>");
		return deled.c_str();
	};
	/**Basic constructor. This is just a basic constructor.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	Primitive(){vertices.clear();polygons.clear();}
};




/**A basic representation of DeleD light. This class contains all infos necessary to create a light and it's connected
* with Vertex and Color class.
*Sample Usage:
\verbatim
--create a Light
Light *mylight=new Light();
mylight->type=LIGHT_TYPE::POINT;
...
\endverbatim
*/
class Light{
public:
	/**Id of this light. \note This id is univoque.*/
	int id;
	/**Type of this Light. Admitted types are : \li \c LIGHT_TYPE::SPOT for a spotlight.\li \c PRIMITIVE_TYPE::DIRECTIONAL for a directional light.
	*\li \c LIGHT_TYPE::POINT for a point light.*/
	String type;
	/**Name of this light. You can set or get this light's name.*/
	String name;
	/**Internal property to hold user info.*/
	String tag;
	/**Internal property to hold user comments.*/
	String comments;
	/**Property that indicates if a light is visible or not.*/
	bool visible;
	/**Property that indicates if a light is active or not.*/
	bool active;
	/**Maximum range of light. Light is visible up to \b range.*/
	int range;
	/**Property that indicates if a light can cast shadows.*/
	bool castShadows;
	/**Snap this light or not.*/
	bool snap;
	/**Intensity of this light.*/
	int intensity;
	/**Diffuse Color of this light.*/
	Color diffuse;
	/**Specular Color of this light.*/
	Color specular;
	/**Constant value, starting point of attenuation. It means that at the position of light you'll have an intensity that has the value of \b constant.*/
	double constant;
	/**Linear value, influences attenuation shape (linear term, I suppose).*/
	double linear;
	/**Quadratic value, influences attenuation shape (quadratic term, I suppose).*/
	double quadratic;
	/**Direction of this light. I'm not sure this must be normalized (divide each member for module of Vertex).*/
	Vertex direction;
	/**Center position of this light.*/
	Vertex position;
	/**Angle of inner cone of light. Angle that forms the inner light cone (more luminous).*/
	int innerangle;
	/**Angle of outer cone of light. Angle that forms the outer light cone (less luminous).*/
	int outerangle;
	/**Falloff value, influences the way lights and shadows are rendered.*/
	int falloff;
	/**Get name property.\return name of this light.*/
	const char * get_name(){return name.c_str();};
	/**Set name property.\param newname name of this light.*/
	void set_name(char *newname){name=String(newname);};
	/**Internal property to hold strings in DeleD format.*/
	String deled;
	/**Light string representation. \return A string representation of Light for use in Deled files and internal callbacks.*/
	const char* to_deled(){
		deled=String("<light id=\"")+toString(id)+String("\" type=\"")+type+String("\" name=\"")+name+String("\" visible=\"")+toString(visible)+String("\" range=\"")+toString(range)+String("\" active=\"")+toString(active)+
			String("\" castShadows=\"")+toString(castShadows)+String("\" snap=\"")+toString(snap)+String("\" intensity=\"")+toString(intensity)+String("\">\n");
		if(tag!=String(""))
			deled=deled+String("<tag>")+tag+String("</tag>\n");
		if(comments!=String(""))
			deled=deled+String("<comments>")+comments+String("</comments>\n");
		deled=deled+String("<diffuse r=\"")+toString(diffuse.r)+String("\" g=\"")+toString(diffuse.g)+String("\" b=\"")+toString(diffuse.b)+String("\" />\n");
		deled=deled+String("<specular r=\"")+toString(specular.r)+String("\" g=\"")+toString(specular.g)+String("\" b=\"")+toString(specular.b)+String("\" />\n");
		deled=deled+String("<position x=\"")+toString(position.x)+String("\" y=\"")+toString(position.y)+String("\" z=\"")+toString(position.z)+String("\" />\n");
		if(type==LIGHT_TYPE::DIRECTIONAL || type==LIGHT_TYPE::SPOT)
		{
			deled=deled+String("<direction x=\"")+toString(direction.x)+String("\" y=\"")+toString(direction.y)+String("\" z=\"")+toString(direction.z)+String("\" />\n");
		}
		if(type==LIGHT_TYPE::SPOT || type==LIGHT_TYPE::POINT)
		{
			deled=deled+String("<attenuation constant=\"")+toString(constant)+String("\" linear=\"")+toString(linear)+String("\" quadratic=\"")+toString(quadratic)+String("\" />\n");
		}
		if(type==LIGHT_TYPE::SPOT)
		{
			deled=deled+String("<cone inner=\"")+toString(innerangle)+String("\" outer=\"")+toString(outerangle)+String("\" falloff=\"")+toString(falloff)+String("\" />\n");
		}
		deled=deled+String("</light>");
		return deled.c_str();};
		/**Standard constructor. Doesn't do much at the moment.*/
		Light(){};
};


/**A complete representation of a DeleD scene. It contains all Primitive, MaterialCategory and Light (PointLight, SpotLight, DirectionalLight) elements.
*It contains some other infos too.
*Sample Usage:\todo add other helper functions.
\verbatim
//create a new scene
Scene *myscene=new Scene();
//loads everything
GetScene(myscene);
\endverbatim
or:
\verbatim
//create a new scene
Scene *myscene=new Scene();
//load scene with all DeleD materials, note that this is necessary
//cause objects depends on materials (uv coords depends on material layers...)
GetMaterials(false,false,myscene);
//load scene with all DeleD primitives
GetPrimitives(false,false,myscene);
//load scene with all DeleD lights
GetLights(false,false,myscene);
...
\endverbatim
*/
class Scene{
public:
	/**Name of this scene. You can set or get name of this scene. \note DeleD doesn't pass (for the moment) this information to plugin system, so remember to add
	*everytime you get a scene from DeleD or everytime you create one.*/
	String name;
	/**Internal property to hold primitive strings in DeleD format.*/
	String deled_primitives;
	/**Internal property to hold settings strings in DeleD format.*/
	String deled_settings;
	/**Internal property to hold material strings in DeleD format.*/
	String deled_materials;
	/**Internal property to hold light strings in DeleD format.*/
	String deled_lights;
	/**Get name property.\return name of this scene.*/
	const char * get_name(){return name.c_str();};
	/**Set name property.\param newname name of this scene.*/
	void set_name(char *newname){name=String(newname);};
	/**Ambient color of this scene. You can set or get color of this scene. This string represents a color in the form \b 00BBGGRR where : \li \b BB is hex value for blue amount (i.e. FF for 255, or 0F for 15).\li \b GG is hex value for green amount (i.e. FF for 255, or 0F for 15).\li \b RR is hex value for red amount (i.e. FF for 255, or 0F for 15).\note DeleD doesn't pass (for the moment) this information to plugin system, so remember to add
	*everytime you get a scene from DeleD or everytime you create one.*/
	Color ambient;
	/**Shadow opacity for this Scene. note DeleD doesn't pass (for the moment) this information to plugin system, so remember to add
	*everytime you get a scene from DeleD or everytime you create one.*/
	unsigned shadow;
	/**Author of this scene.*/
	String author;
	/**Comments on this scene.*/
	String comments;
	/**A vector containing all Material elements of this scene. Note that you can use all standard vector methods.*/
	std::vector<MaterialCategory> materials;
	/**Number of MaterialCategory in this scene. \return Number of MaterialCategory in this scene material list.*/
	int getMaterialsSize(){return materials.size();};
	/**Add a MaterialCategory at the end of current material list. \param materia MaterialCategory to add.*/
	void addMaterialCategory(MaterialCategory materia){materials.push_back(materia);};
	/**Remove MaterialCategory at a particular position in list. \param index index of MaterialCategory to remove from current materials list. \note materials list is
	* a zero based list, so position \b 0 will be the first one, and the last is \b num_materials-1.*/
	void removeMaterialCategory(int index){std::vector<MaterialCategory>::iterator p=materials.begin(); if(index<materials.size() && index>-1){p+=index;materials.erase(p);}};
	/**Get a MaterialCategory in materials list to modify it. \param index index of MaterialCategory to obtain from current materials list. \return MaterialCategory wanted so you can modify it. \note If you get a MaterialCategory
	* with this function then each modification will be automatically reported in MaterialCategory in materials list (in lua too), cause you'll get just a pointer to an existent MaterialCategory.*/
	MaterialCategory *getMaterialCategory(int index){return &materials[index];};
	/**A vector containing all Primitive elements of this scene. Note that you can use all standard vector methods.*/
	std::vector<Primitive> primitives;
	/**Number of Primitive in this scene. \return Number of Primitive in this scene objects list.*/
	int getPrimitivesSize(){return primitives.size();};
	/**Add a Primitive at the end of current Primitive list. \param obj Primitive to add.*/
	void addPrimitive(Primitive obj){primitives.push_back(obj);};
	/**Remove Primitive at a particular position in list. \param index index of Primitive to remove from current objects list. \note primitives list is
	* a zero based list, so position \b 0 will be the first one, and the last is \b num_objects-1.*/
	void removePrimitive(int index){std::vector<Primitive>::iterator p=primitives.begin(); if(index<primitives.size() && index>-1){p+=index;primitives.erase(p);}};
	/**Get a Primitive in primitives list to modify it. \param index index of Primitive to obtain from current objects list. \return Object wanted so you can modify it. \note If you get a Primitive
	* with this function then each modification will be automatically reported in Primitive in primitives list (in lua too), cause you'll get just a pointer to an existent Primitive.*/
	Primitive *getPrimitive(int index){return &primitives[index];};
	/**A vector containing all Light elements of this scene. Note that you can use all standard vector methods.*/
	std::vector<Light> lights;
	/**Number of Light in this scene. \return Number of Light in this scene lights list.*/
	int getLightsSize(){return lights.size();};
	/**Add a Light at the end of current lights list. \param obj Light to add.*/
	void addLight(Light obj){lights.push_back(obj);};
	/**Remove Light at a particular position in list. \param index index of Light to remove from current lights list. \note lights list is
	* a zero based list, so position \b 0 will be the first one, and the last is \b num_lights-1.*/
	void removeLight(int index){std::vector<Light>::iterator p=lights.begin(); if(index<lights.size() && index>-1){p+=index;lights.erase(p);};};
	/**Get a Light in lights list to modify it. \param index index of Light to obtain from current lights list. \return Light wanted so you can modify it. \note If you get a Light
	* with this function then each modification will be automatically reported in Light in lights list (in lua too), cause you'll get just a pointer to an existent Light.*/
	Light *getLight(int index){return &lights[index];};
	/**Get the highest id of primitives contained in this scene.\return highest id.*/
	int getPrimitiveHighestId(){
		int id=0;
		for(int i=0;i<primitives.size();i++)
			if(primitives[i].id>id)
				id=primitives[i].id;
		return id;
	};
	/**Get the highest id of materials contained in this scene.\return highest id.*/
	int getMaterialHighestId(){
		int id=0;
		for(int i=0;i<materials.size();i++)
			for(int j=0;j<materials[i].materials.size();j++)
				if(materials[i].materials[j].id>id)
					id=materials[i].materials[j].id;
		return id;
	};
	/**Get the highest id of lights contained in this scene.\return highest id.*/
	int getLightHighestId(){
		int id=0;
		for(int i=0;i<lights.size();i++)
			if(lights[i].id>id)
				id=lights[i].id;
		return id;
	};
	/**Primitive string representation. \return A string representation of Primitive in this scene for use in Deled internal callbacks.\note This function is not suitable to be used for DeleD file writing cause it lacks new line characters, it's used just to pass data to DeleD internally with functions.*/
	const char * to_deled_primitives(){deled_primitives=String("<primitives highestID=\"")+toString(getPrimitiveHighestId())+String("\">\n");
	for(int i=0;i<primitives.size();i++)
		deled_primitives=deled_primitives+String(primitives[i].to_deled())+String("\n");
	deled_primitives=deled_primitives+String("</primitives>");return deled_primitives.c_str();
	};
	/**Materials string representation. \return A string representation of Materials in this scene for use in Deled internal callbacks.\note This function is not suitable to be used for DeleD file writing cause it lacks new line characters, it's used just to pass data to DeleD internally with functions.*/
	const char * to_deled_materials(){deled_materials=String("<materials highestID=\"")+toString(getMaterialHighestId())+String("\">\n");
	for(int i=0;i<materials.size();i++)
		deled_materials=deled_materials+String(materials[i].to_deled())+String("\n");
	deled_materials=deled_materials+String("</materials>");return deled_materials.c_str();
	};
	const char * to_deled_settings(){deled_settings=String("<settings name=\"")+name+String("\" author=\"")+author+String("\" comments=\"")+comments+String("\" shadowOpacity=\"")+toString(shadow)+String("\">\n");
	deled_settings=deled_settings+String("<ambient r=\"")+toString(ambient.r)+String("\" g=\"")+toString(ambient.g)+String("\" b=\"")+toString(ambient.b)+String("\" />\n</settings>");return deled_settings.c_str();
	};
	/**Light string representation. \return A string representation of Light in this scene for use in Deled internal callbacks.\note This function is not suitable to be used for DeleD file writing cause it lacks new line characters, it's used just to pass data to DeleD internally with functions.*/
	const char * to_deled_lights(){
		deled_lights=String("<lights highestID=\"")+toString(getLightHighestId())+String("\">\n");
		for(int i=0;i<lights.size();i++)
			deled_lights=deled_lights+String(lights[i].to_deled())+String("\n");
		deled_lights=deled_lights+String("</lights>");
		return deled_lights.c_str();
	};
	/**Basic constructor. Doesn't do much at the moment.
	*\todo Initialize all elements to default values, for now you need to do manually.
	*/
	Scene(){materials.clear();primitives.clear();lights.clear();};
};


/*!relatesalso Scene
*This function is internally used by GetMaterials(bool,bool,Scene) to obtain
*wanted data from strings, please don't call directly.*/
void  GetMaterials( String materials_data, bool onlyid, Scene *scene );

/*!relatesalso Scene
*This function is internally used by GetLights(bool,bool,Scene) to obtain
*wanted data from strings, please don't call directly.*/
void GetLights(String lights_data, bool onlyid, Scene *scene);

/*!relatesalso Scene
*This function is internally used by GetObjects(bool,bool,Scene) to obtain
*wanted data from strings, please don't call directly.*/
void GetPrimitives( String objects_data, bool onlyid, Scene *scene );

/*!\relatesalso Scene
*Retrieve all materials from DeleD copying to a scene variable. This function get all materials ,or just selected ones from DeleD and put them in a precedently created Scene variable.
*\param selected if this is set to \c true you'll get only selected materials, else if it's set to \c false you'll get all materials.
*\param onlyid if this is set to \c true you'll get only id of materials, else if it's set to \c false you'll get everything.
*\param scene a Scene where you must put materials, scene must exist, or function will fail.
*\note A scene must exist, you can create with : \code Scene *myscene = new Scene();\endcode, once you created a scene you can use it with this and others getting functions, but note that materials are just added to current list.
\verbatim
//create a new scene
Scene *myscene=new Scene();
//load scene with all DeleD materials
GetMaterials(false,false,myscene);
//load scene with all DeleD objects
GetPrimitives(false,false,myscene);
//load scene with all DeleD lights
GetLights(false,false,myscene);
...
\endverbatim
*/
void GetMaterials(bool selected,bool onlyid,Scene *scene);

/*!\relatesalso Scene
*Retrieve all primitives from DeleD copying to a scene variable. This function get all objects ,or just selected ones from DeleD and put them in a precedently created Scene variable.
*\param selected if this is set to \c true you'll get only selected primitives, else if it's set to \c false you'll get all primitives.
*\param onlyid if this is set to \c true you'll get only id of primitives, else if it's set to \c false you'll get everything.
*\param scene a Scene where you must put objects, scene must exist, or function will fail.
*\note A scene must exist, you can create with : \code Scene *myscene = new Scene();\endcode, once you created a scene you can use it with this and others getting functions, but note that objects are just added to current list.
\verbatim
//create a new scene
Scene *myscene=new Scene();
//load scene with all DeleD materials
GetMaterials(false,false,myscene);
//load scene with all DeleD objects
GetPrimitives(false,false,myscene);
//load scene with all DeleD lights
GetLights(false,false,myscene);
...
\endverbatim
*/
void GetPrimitives(bool selected,bool onlyid,Scene *scene);

/*!\relatesalso Scene
*Retrieve all lights from DeleD copying to a scene variable. This function get all ligths ,or just selected ones from DeleD and put them in a precedently created Scene variable.
*\param selected if this is set to \c true you'll get only selected lights, else if it's set to \c false you'll get all lights.
*\param onlyid if this is set to \c true you'll get only id of lights, else if it's set to \c false you'll get everything.
*\param scene a Scene where you must put lights, scene must exist, or function will fail.
*\note A scene must exist, you can create with : \code Scene *myscene = new Scene();\endcode, once you created a scene you can use it with this and others getting functions, but note that lights are just added to current lists. And remember that lights are put in
* different lists (each for each type of light), see Scene for more details.
\verbatim
//create a new scene
Scene *myscene=new Scene();
//load scene with all DeleD materials
GetMaterials(false,false,myscene);
//load scene with all DeleD objects
GetPrimitives(false,false,myscene);
//load scene with all DeleD lights
GetLights(false,false,myscene);
...
\endverbatim
*/
void GetLights(bool selected,bool onlyid,Scene *scene);


/*!relatesalso Scene
*This retrieve the current complete scene from DeleD, it practically
*retrieve all materials,objects and lights and copy to the Scene wanted.
*\note A scene must exist, you can create with : \code Scene *myscene = new Scene();\endcode, once you created a scene you can use it with this and others getting functions, but note that lights are just added to current lists. And remember that lights are put in
* different lists (each for each type of light), see Scene for more details.
\verbatim
//create a new scene
Scene *myscene=new Scene();
//load complete scene from DeleD
GetScene(myscene);
...
\endverbatim
*/
void  GetScene(Scene *scene );


/*!\relatesalso Scene
*Set all materials of this scene variable to DeleD. This function sets all materials from a Scene variable to DeleD.
*\param scene a Scene which materials will be copied to DeleD, scene must exist, or function will fail.
*\note A scene must exist, you can create with : \code Scene *myscene = new Scene();\endcode and some materials must be in it.
\verbatim
//suppose we have a scene called myscene,full of objects,materials and lights we want to pass to DeleD
//set materials provided by this scene to DeleD
SetMaterials(myscene);
//set objects provided by this scene to DeleD
SetObjects(myscene);
//set lights provided by this scene to DeleD
SetLights(myscene);
...
\endverbatim
*/
void SetMaterials(Scene* scene);

/*!\relatesalso Scene
*Set all objects of this scene variable to DeleD. This function sets all objects from a Scene variable to DeleD.
*\param scene a Scene which objects will be copied to DeleD, scene must exist, or function will fail.
*\note A scene must exist, you can create with : \code Scene *myscene = new Scene();\endcode and some objects must be in it.
\verbatim
//suppose we have a scene called myscene,full of objects,materials and lights we want to pass to DeleD
//set materials provided by this scene to DeleD
SetMaterials(myscene);
//set objects provided by this scene to DeleD
SetPrimitives(myscene);
//set lights provided by this scene to DeleD
SetLights(myscene);
...
\endverbatim
*/
void SetPrimitives(Scene *scene);


/*!\relatesalso Light
*Add a PointLight to DeleD. This function adds wanted pointlight to DeleD.
*\param light a PointLight to be copied to DeleD, PointLight must exist, or function will fail.
*\param highestID Lights highest id.
*\note A pointlight must exist, you can create with : \code PointLight *mylight = new PointLight();\endcode and its values must be set.
\verbatim
//suppose we have a PointLight called mylight we want to pass to DeleD
//set light provided to DeleD
SetLights(mylight,9);
...
\endverbatim
*/
void SetLights(Light *light,int highestID);



/*!\relatesalso Material
*Add a Material to DeleD. This function adds wanted material to DeleD.
*\param materia a Material to be copied to DeleD, Material must exist, or function will fail.
*\param category Category where material is added.
*\param highestID Materials highest id.
*\note A material must exist, you can create with : \code Material *mymaterial = new Material();\endcode and its values must be set.
\verbatim
//suppose we have a Material called mymaterial we want to pass to DeleD
//set material provided to DeleD
SetMaterials(mymaterial,String("System"),9);
...
\endverbatim
*/
void SetMaterials(Material* materia,String category,int highestID);

/*!\relatesalso Primitive
*Add a Object to DeleD. This function adds wanted object to DeleD.
*\param obj a Primitive to be copied to DeleD, Primitive must exist, or function will fail.
*\param highestID Primitives highest id.
*\note A object must exist, you can create with : \code Primitive *myobj = new Primitive();\endcode and its values must be set.
\verbatim
//suppose we have a Primitive called myobj we want to pass to DeleD
//set object provided to DeleD
SetPrimitives(myobj,9);
...
\endverbatim
*/
void SetPrimitives(Primitive *obj,int highestID);

/*!This function returns a Primitive sample cube in DeleD.
*\param id ID of cube created.*/
Primitive CreateDeleDCube(int id);

/**This function returns the highest id of materials contained in actual DeleD scene.\return highest id.*/
int GetMaterialsHighestID();

/**This function returns the highest id of primitives contained in actual DeleD scene.\return highest id.*/
int GetPrimitivesHighestID();

/**This function returns the highest id of lights contained in actual DeleD scene.\return highest id.*/
int GetLightsHighestID();
[!else]

/**This function is internally used to obtain
*wanted data strings.\param selected If true you'll get selected materials only.\param onlyid If true you'll get ID only.\return Materials string.*/
String  GetMaterials(bool selected, bool onlyid);

/**This function is internally used to obtain
*wanted data strings.\param selected If true you'll get selected lights only.\param onlyid If true you'll get ID only.\return Lights string.*/
String GetLights(bool selected, bool onlyid);

/**This function is internally used to obtain
*wanted data strings.\param selected If true you'll get selected primitives only.\param onlyid If true you'll get ID only.\return Primitives string.*/
String GetPrimitives(bool selected, bool onlyid);

/**This function returns the highest id of materials contained in actual DeleD scene.\return highest id.*/
int GetMaterialsHighestID();

/**This function returns the highest id of primitives contained in actual DeleD scene.\return highest id.*/
int GetPrimitivesHighestID();

/**This function returns the highest id of lights contained in actual DeleD scene.\return highest id.*/
int GetLightsHighestID();

/*!This function returns a Primitive sample cube in DeleD.
*\param id ID of cube created.*/
String CreateDeleDCube(int id);

/**This function is internally used to pass
*wanted strings to DeleD.\param data Data to pass.*/
void SetData(String data);
[!endif]
#endif /* _SUPPORT_H_ */
